@@ -5,13 +5,17 @@
 #  /usr/share/dictionaries-common for single ispell dicts/wordlists use
 # ---------------------------------------------------------------------------
 
+use strict;
+
 sub dico_get_packages (){
   # Get list of packages sharing the question
   my $class    = shift;
   my $question = "shared/packages-$class";
+  my @pkglist  = ();
 
   my ($errorcode,$packages) = metaget ($question, "owners");
-  return [ split (/\s*,\s*/, $packages) ] unless $errorcode;
+  @pkglist = split (/\s*,\s*/, $packages) unless $errorcode;
+  return \@pkglist;
 }
 
 sub dico_parse_languages (){
@@ -25,7 +29,7 @@ sub dico_parse_languages (){
 
   $packages = &dico_get_packages($class) unless $packages;
 
-  foreach $pkg ( @$packages ){
+  foreach my $pkg ( @$packages ){
     my ($errorcode, $entry ) = metaget("$pkg/$variant", "default");
     unless ( $errorcode ){
       $entry =~ s/^\s+//;
@@ -56,7 +60,7 @@ sub dico_get_all_choices (){
     print STDERR " * Packages with elanguages: $elanglist\n" if $debug;
   }
 
-  foreach $pkg ( keys %$languages ){
+  foreach my $pkg ( keys %$languages ){
     my @langs  = split(/\s*,\s*/, $languages->{$pkg});
     my @elangs = @langs;
     if ( exists $elanguages->{$pkg} ){
@@ -67,7 +71,7 @@ sub dico_get_all_choices (){
       }
       @elangs = @tmp if ( $#langs == $#tmp );
     }
-    foreach $index ( 0 .. $#langs ){
+    foreach my $index ( 0 .. $#langs ){
       $mappinghash{$langs[$index]} = $elangs[$index];
     }
   }
@@ -76,58 +80,74 @@ sub dico_get_all_choices (){
 		      sort {lc($mappinghash{$a}) cmp lc($mappinghash{$b})}
 		      keys %mappinghash);
   if ( $debug ){
-    print STDERR "- Choices:\n[$choices]\n";
-    print STDERR "- Echoices:\n[$echoices]\n";
+    print STDERR " * Choices:\n   [$choices]\n";
+    print STDERR " * Echoices:\n   [$echoices]\n";
     print STDERR "-------- $debug_prefix end --------\n";
   }
   return $choices, $echoices;
 }
 
 sub dc_debconf_select (){
-  my $class       = shift;
-  my $priority    = shift;
-  my $question    = "dictionaries-common/default-$class";
-  my $packages    = &dico_get_packages($class);
+  my $classinfo   = shift;
   my $debug       = 1 if exists $ENV{'DICT_COMMON_DEBUG'};
   my $reconfigure = 1 if exists $ENV{'DEBCONF_RECONFIGURE'};
-  my $flagdir     = "/var/cache/dictionaries-common";
-  my $newflag     = "$flagdir/flag-$class-new";
-  my $langscript  = "/usr/share/dictionaries-common/dc-debconf-default-value.pl";
   my $echoices;
-  my @oldchoices  = ();
-  my %newchoices  = ();
   my %title       = ('ispell'   => "Dictionaries-common: Ispell dictionary",
 		     'wordlist' => "Dictionaries-common: Wordlist dictionary"
-		     );
-  my $debug_prefix = "[$class,dc_debconf_select]";
+    );
 
+  my $class;
+  my $priority;
+  my $is_dcconfig;
+  # If $classinfo is a hash reference, function is called from dictionaries-common.config
+  if ( ref($classinfo) eq 'HASH' ){
+    $class       = $classinfo->{'class'};
+    $priority    = $classinfo->{'priority'} if ( defined $classinfo->{'priority'} );
+    $is_dcconfig = 1;
+  } else {
+    # Otherwise is called from ispell dictionary/wordlist config
+    $class = $classinfo;
+  }
+
+  my $packages     = &dico_get_packages($class);
   return unless $packages;
 
+  my $question     = "dictionaries-common/default-$class";
+  my $flagdir      = "/var/cache/dictionaries-common";
+  my $newflag      = "$flagdir/flag-$class-new";
+  my $debug_prefix = "[$class,dc_debconf_select]";
+
   print STDERR "----- $debug_prefix start -----------\n" if $debug;
 
   # Get new base list of provided languages
+  my %newchoices  = ();
   my $languages = &dico_parse_languages($class,"languages",$packages);
-  foreach $pkg ( keys %$languages ) {
-    foreach $lang ( split(/\s*,\s*/, $languages->{$pkg}) ){
+  foreach my $pkg ( keys %$languages ) {
+    foreach my $lang ( split(/\s*,\s*/, $languages->{$pkg}) ){
       $newchoices{$lang}++;
     }
   }
   my $choices = join (', ', sort {lc($a) cmp lc($b)} keys %newchoices);
 
   # Get old list of provided languages
-  @oldchoices = split(/\s*,\s*/,metaget ($question, "choices-c"));
+  my @oldchoices  = split(/\s*,\s*/,metaget ($question, "choices-c"));
   pop @oldchoices;            # Remove the manual entry
   my $oldchoices = join (', ', sort {lc($a) cmp lc($b)} @oldchoices);
 
   # If dictionaries-common is already installed (-r $langscript),
-  # there are elements for this chass to be installed (%newchoices)
+  # there are elements for this class to be installed (%newchoices)
   # and there were none before (! $oldchoices), means that we are installing
   # for the first time elements in this class, with dictionaries-common
   # already installed. Try getting a reasonable default value
+  my $langscript  = "/usr/share/dictionaries-common/dc-debconf-default-value.pl";
   if ( -r $langscript && %newchoices && ! $oldchoices ){
-    print STDERR "$debug_prefix: Configuring $class class for the first time\n\n" if $debug;
-    require $langscript;
-    &dc_set_default_value_for_class($class);
+    print STDERR "$debug_prefix: Configuring class \"$class\" for the first time\n\n" if $debug;
+    # If called from dictionaries-common.config we already have
+    # $langscript, and probably more recent. Including it here will cause
+    # some warnings about subroutine re-definitions and even errors.
+    require $langscript unless $is_dcconfig;
+    my $guessed = &dc_set_default_value_for_class($class);
+    $priority = $guessed->{'priority'} if ( defined $guessed->{'priority'} );
   }
 
   # Read current value of default ispell dict / wordlist.
@@ -157,10 +177,10 @@ sub dc_debconf_select (){
 
   print STDERR
     "$debug_prefix:\n" .
-    " Class: $class, Priority: $priority\n" .
-    " Question: $question, Previous or guessed value: $curval\n" .
-    " New choices:[$choices]\n" .
-    " Old choices:[$oldchoices]\n" if $debug;
+    " * Class: $class, Priority: $priority\n" .
+    " * Question: $question, Previous or guessed value: $curval\n" .
+    " * New choices: [$choices]\n" .
+    " * Old choices: [$oldchoices]\n" if $debug;
 
   # May ask question if there is no match
   if ( scalar %newchoices ) {
@@ -168,7 +188,7 @@ sub dc_debconf_select (){
       fset ($question, "seen", "false");
       # Let future processes in this apt run know that a new $class element is to be installed
       if ( -d $flagdir ) {
-	open ($FLAG,"> $newflag")
+	open (my $FLAG, "> $newflag")
 	  or die "Could not open $newflag for write. Aborting ...\n";
 	print $FLAG "1\n";
 	close $FLAG;
@@ -187,10 +207,10 @@ sub dc_debconf_select (){
   }
 
   # If called from dictionaries-common.config, check actual values in debug mode
-  if ( $debug && $fromdcconfig ){
-    print STDERR "dictionaries-common.config: Checking really set values for $question\n";
-    print STDERR " Choices-C string: " . metaget ($question, "choices-c") . "\n";
-    print STDERR " Really set value: " . get ($question) . "\n";
+  if ( $debug && $is_dcconfig ){
+    print STDERR " * Checking really set values for $question:\n";
+    print STDERR "   - Choices-C string: " . metaget ($question, "choices-c") . "\n";
+    print STDERR "   - Really set value: " . get ($question) . "\n";
   }
   print STDERR "----- $debug_prefix end -----------\n" if $debug;
 }
